<?php
namespace FiloBlu\Flow\Model\From;

class Product extends AbstractFrom {
    
    /**
    * @var Monolog\Logger
    */
    protected $_logger;

    /**
     * @param \Magento\Framework\Model\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param \Magento\Framework\Model\ResourceModel\AbstractResource $resource
     * @param \Magento\Framework\Data\Collection\Db $resourceCollection
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \FiloBlu\Flow\Helper\Product $helper_product,
        \FiloBlu\Flow\Helper\Data $helper_flow,
        \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Catalog\Model\ProductFactory $productFactory,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        array $data = []
    ) {
        $this->helper_product = $helper_product;
        $this->helper_flow = $helper_flow;
        $this->storeManager = $storeManager;
        $this->productFactory = $productFactory;
        $this->_logger = $loggerProvider->getLogger();
        parent::__construct($context, $registry, $resource, $resourceCollection, $data);
    }

    /**
     * @return void
     */
    public function _construct()
    {
        $this->_init('FiloBlu\Flow\Model\ResourceModel\From\Product');
    }
    

    public function process() {

        $ProdHelper = $this->helper_product;

        $attributeSetId = $ProdHelper->getProductAttributeSet();
        if (empty($attributeSetId)) {
            $message = sprintf("Product attribute_set not defined!");
            $this->error(true, $message);
            $this->_logger->info($message);
            return $this;
        }

        $sku_configurabile = $this->getCodiceConfigurabile();

        $TaxClassCode = $this->getFasciaIvaArticoli();
        $mappedTaxClassId = $ProdHelper->getMappedTaxClassId($TaxClassCode);
        if (!$mappedTaxClassId) {
            $message = sprintf("tax_class_code '%s' has no Magento tax class associated.", $TaxClassCode);
            $this->error(true, $message);
            $this->_logger->info($message);
            return $this;
        }

        $productSku = $this->helper_flow->removeTextSpaces($this->getCodiceArticolo());
        if (empty($productSku)) {
            return $this->error(false, "'codice_articolo' is empty.");
        }

        $model_product = \Magento\Framework\App\ObjectManager::getInstance()->create('Magento\Catalog\Model\Product');

        $product_id = $model_product->getIdBySku($productSku);

        if (!$product_id) {

            if (!$ProdHelper->createNewProducts()) {
                $message = sprintf('The product with SKU %s has not been found and the module is configured to not create it', $skuFilter);
                $this->error(true, $message);
                $this->_logger->info($message);
                return $this;
            }

            $product = $model_product
                ->setTypeId(\Magento\Catalog\Model\Product\Type::TYPE_SIMPLE)
                ->setAttributeSetId($attributeSetId)
                ->setStatus($ProdHelper->getInitialProductStatus())
                ->setVisibility(\Magento\Catalog\Model\Product\Visibility::VISIBILITY_BOTH)
                ->setStoreId(0)
                ->setDescription($this->getNomeArticolo())
                ->setShortDescription($this->getNomeArticolo())
                ->setSku($productSku)
                ->setUrlKey(strtolower(preg_replace('#[^0-9a-z]+#i', '-', $this->getNomeArticolo().'-'.$productSku)))
                ->setPrice(0)
                ->setPriceView(0)
                ->setWeight(0)
                ->setName($this->getNomeArticolo());

            if ($ProdHelper->getAddToWebsites()) {

                $websites = $this->storeManager->getWebsites();

                foreach ($websites as $website) {
                    $websitesIds[] = $website->getId();
                }
                $product->setWebsiteIds($websitesIds);
            }

        }

        $productAttributesData = array(
            "product_sku" => $this->getCodiceArticolo(),
            "tax_class_id" => $mappedTaxClassId,
            "parent_sku" => $sku_configurabile
        );

        //initial stock configuration
        $stockData = $this->getInitialStockData();

        if (!$product_id) {
            //setting the initial stock config...
            $productAttributesData["stock_data"] = $stockData;
            //saving the product...
            $product->addData($productAttributesData);
            try{
                $product->save();
            }catch(\Exception $e){
                $message = $e->getMessage();
                $this->_logger->info($message);
            }

        } else {

            // da implementare
            if (true or !$this->getVisibilitaSuStore()) {
                $productAttributesData["status"] = \Magento\Catalog\Model\Product\Attribute\Source\Status::STATUS_ENABLED;
            }

            //saving attributes...(ERRORE IN UPDATE PRODOTTO)
            \Magento\Framework\App\ObjectManager::getInstance()->get('Magento\Catalog\Model\Product\Action')->updateAttributes(array($product_id), $productAttributesData, 0);

            //saving stock configuration...
            $stockItem = \Magento\Framework\App\ObjectManager::getInstance()->create('Magento\CatalogInventory\Api\StockRegistryInterface')->getStockItem($product_id,0);

            if ( $product_id != \Magento\Catalog\Model\Product\Type::TYPE_BUNDLE && ( $stockData['backorders'] != \Magento\CatalogInventory\Model\Stock::BACKORDERS_NO &&
                    ($stockData['min_qty'] == 0 || $stockItem->getQty() > $stockData['min_qty'])
                ) ) {
                $stockItem->setIsInStock(true);
            } elseif( $product_id != \Magento\Catalog\Model\Product\Type::TYPE_BUNDLE ) {
                if($stockItem->getQty() >= $stockItem->getMinSaleQty()){
                    $stockItem->setIsInStock(true);
                }else{
                    $stockItem->setIsInStock(false);
                }
            }

            $stockItem->addData($stockData)->save();

        }

        return $this;
    }

    public function getInitialProductStatus() {
        $h = Mage::helper('fbflow/product');
        $status = \Magento\Catalog\Model\Product\Attribute\Source\Status::STATUS_DISABLED;
        if ($h->useSupplierProductStatus()) {
            if (!$this->getVisibilitaSuStore()) {
                $status = \Magento\Catalog\Model\Product\Attribute\Source\Status::STATUS_DISABLED;
            } else {
                $status = \Magento\Catalog\Model\Product\Attribute\Source\Status::STATUS_ENABLED;
            }
        } else {
            $status = $h->getDefaultProductStatus();
        }

        return $status;
    }

    public function getInitialStockData() {
        $stockData = array(
            'min_sale_qty' => $this->getQtaMinimaVendibile(),
            'max_sale_qty' => $this->getQtaMassimaVendibile(),
            'qty' => 1,
            'backorders' => $this->getVendibilitaSenzaStock() ?
                \Magento\CatalogInventory\Model\Stock::BACKORDERS_YES_NONOTIFY :
                \Magento\CatalogInventory\Model\Stock::BACKORDERS_NO
        );

        // when no backorders - reset the min_qty
        $stockData["min_qty"] = 0;
        $stockData["use_config_min_qty"] = true;
        return $stockData;
    }




}